#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "sim.c.trap"

#ifndef SIM_TRAP
  /* focus areas */
  #define NFOCUS	3
  int focus[NFOCUS][3] = {{30, 60, 80}, {00, 20, 50}, {50, 90, 25}};
  int fw[NFOCUS] = {30, 20, 50};	/* focus weights */
  int fc=0;		/* current focus */
#endif

/* probability profile */
#ifndef SIM_TRAP
  #define PDIR	60	/* maintain direction */
#endif
#define PUFO	10	/* use focus */
#define PCFO	05	/* change focus */
#define CHOOSE	random()%100

/* delay profile */
#define D1	100
#define D10	1000
#define D100	5000

/* simulation length */
#define NSTEPS	100

/* navigation vars */
int op=0;		/* current operation: 0 level, 1 left, 2 right, ... */
int c_time=0;		/* current time */
int xt, yt, lt;		/* temporary focus */
int xc=25, yc=75, lc=0;	/* current window */

sgn(int n)
{
  return n<0 ? -1 : 1;
}

exp_neg(int n)
{
  double m = (double) (random()%n);
  double y = m*m*m/(double)(n*n);
  return (int)y;
}

void choose_dir()
{
  if (CHOOSE >= PDIR) op = op ? 0 : 1 + (random()%2);
}

void choose_foc()
{
  int old_focus = fc;

  while ((fc = (random()%NFOCUS)) == old_focus);
}

void proj_focus()
{
  if (op) {xt=focus[fc][0]; yt=focus[fc][1]; lt=lc;}
  else {xt=xc; yt=yc; lt=focus[fc][2];}
}

void adjust(int *n)
{
  if (*n<0) *n=0;
  if (*n>100) *n=100;
}

void gen_pseudo()
{
  do {	lt = lc + exp_neg(100)*sgn(2*(random()%2)-1);
	adjust(&lt);
  } while (lt==lc);

  do {	xt = xc + exp_neg(100)*sgn(2*(random()%2)-1);
	adjust(&xt);
	yt = yc + exp_neg(100)*sgn(2*(random()%2)-1);
	adjust(&yt);
  } while (xt>=yt || xt==xc && yt==yc);

  if (op) {lt=lc;}
  else {xt=xc; yt=yc;}
}

void print_one(int sched_time, int o, int x, int y, int l)
{
  printf("%d\t %d %4.2f\n", sched_time, o, 
	o==1 ? x/100. : (o==2 ? y/100. : l/100.));
  fflush(stdout);
}

main()
{
  int DELTA, delta, dir, i, first_time;
  int type;				/* type when side operation:
					   0 - both handlers -> xt
					   1 - both handlers -> yt
					   2 - left handler  -> xt
					   3 - right handler -> yt */
  time_t seed;

  time(&seed);
  srandom((unsigned) seed);

  for (i=0; i<NSTEPS; i++)
  {
	choose_dir();

	if (CHOOSE < PUFO)	/* use focus */
	{
		if (CHOOSE < PCFO)	/* change current focus */
		{
			DELTA = D100;
			choose_foc();
		}
		else			/* same focus */
		{
			DELTA = D10;
		}

		proj_focus();
	}
	else			/* generate pseudo-focus */
	{
		DELTA = D10;
		gen_pseudo();
	}

	type = random()%4;
	delta=5-exp_neg(5);
	first_time = 1;

/*
	printf("xc=%2d yc=%2d lc=%2d    xt=%2d yt=%2d lt=%2d\n",
		xc, yc, lc, xt, yt, lt);
*/

	if (!op) for(dir=lc<lt ;;)
	{
		if (dir && lc+delta>lt || !dir && lc-delta<lt) break;
		lc += dir ? delta : -delta;
		c_time += first_time?DELTA:D1;
		print_one(c_time, 3, xc, yc, lc);
		first_time=0;
	}
	else if (type==0) for(dir=xc<xt ;;)
	{
		if (dir && (xc+delta>xt || yc+delta>100) || 
		   !dir && (xc-delta<xt || xc-delta< 0)) break;
		xc += dir ? delta : -delta;
		yc += dir ? delta : -delta;
		c_time += first_time?DELTA:D1;
		print_one(c_time, 1+dir, xc, yc, lc);
		print_one(c_time, 2-dir, xc, yc, lc);
		first_time=0;
	}
	else if (type==1) for(dir=yc<yt ;;)
	{
		if (dir && (yc+delta>yt || yc+delta>100) ||
		   !dir && (yc-delta<yt || xc-delta< 0)) break;
		xc += dir ? delta : -delta;
		yc += dir ? delta : -delta;
		c_time += first_time?DELTA:D1;
		print_one(c_time, 1+dir, xc, yc, lc);
		print_one(c_time, 2-dir, xc, yc, lc);
		first_time=0;
	}
	else if (type==2) for(dir=xc<xt ;;)
	{
		if (dir && (xc+delta>xt || xc+delta>yc)  ||
		   !dir && (xc-delta<xt || xc-delta< 0)) break;
		xc += dir ? delta : -delta;
		c_time += first_time?DELTA:D1;
		print_one(c_time, 1, xc, yc, lc);
		first_time=0;
	}
	else if (type==3) for(dir=yc<yt ;;)
	{
		if (dir && (yc+delta>yt || yc+delta>100) ||
		   !dir && (yc-delta<yt || yc-delta<xc)) break;
		yc += dir ? delta : -delta;
		c_time += first_time?DELTA:D1;
		print_one(c_time, 2, xc, yc, lc);
		first_time=0;
	}
  }

  print_one(c_time+D100, 5, 0, 0, 0);
}

